-- Informatics 1 - Functional Programming 
-- Tutorial 7
--
-- Week 9 - Due: 20/21 Nov.

module Tutorial7 (hilbert)
	where

import LSystem
import Test.QuickCheck

-- Exercise 1

-- 1a. split
split :: Command -> [Command]
split (p :#: q) = (split p) ++ (split q)
split (Sit) = []
split (p) = [p]  


-- 1b. join
join :: [Command] -> Command
join (x:[]) = x
join (x:xs) = x :#: (join xs)  

--1c. testing join, split
prop_join_split :: Command -> Bool
prop_join_split c = split ( join ( split c)) == split c



-- Exercise 2
-- 2a. copy
copy :: Int -> Command -> Command
copy n cmd = join [cmd | x <- [1..n]]

copy' :: Int -> Command -> Command
copy' n cmd = Sit

-- 2b. pentagon
-- 2b. pentagon
pentagon :: Distance -> Command
pentagon d = copy 5 (Go d :#: Turn 72)

-- 2c. polygon
polygon :: Distance -> Int -> Command
polygon d sides = copy sides (Go d :#: Turn (360/fromIntegral(sides)))



-- Exercise 3
-- spiral
spiral :: Distance -> Int -> Distance -> Angle -> Command
spiral _ 0 _ _ = Sit
spiral side n step angle = Go side :#: Turn angle :#: (spiral (side + step) (n-1) step angle)


-- Exercise 4
-- optimise
optimise :: Command -> Command
optimise cmd = join $ optimiseLoop $ split cmd
               where
                 optimiseLoop x = if x==x' then x else (optimiseLoop x')
                                  where
                                    x' = optimiseStep x

                                    optimiseStep [] = []
                                    optimiseStep (Sit : path) = optimiseStep path
                                    optimiseStep ((Go 0): path) = optimiseStep path
                                    optimiseStep ((Go x):(Go y):path) = optimiseStep ((Go (x+y)):path)
                                    optimiseStep ((Go x):path) = (Go x):(optimiseStep path)
                                    optimiseStep ((Turn 0):path) = optimiseStep path
                                    optimiseStep ((Turn x):(Turn y):path) = optimiseStep ((Turn (x + y)):path)
                                    optimiseStep ((Turn x):path) = (Turn x):(optimiseStep path)



-- L-Systems
{-
Data Lexp = F Lexp
          | G Lexp
          | + Lexp
          | - Lexp
          | [] Lexp
          | Arrow Lexp Lexp

Data Rule = Arrow Lexp Lexp

-}
{-
fromLtoCommand :: Angle -> Start -> Rewrite -> Distance -> Command
fromLtoCommand angle start rewrite distance = f x 
                                       where
                                         f 0 = Go x
                                         f (x+1) = 
-}

simple :: Int -> Angle -> Command
simple x angle = f x
                 where
                   f 0 = Go 10
                   f (x+1) = p :#: (f x) :#: n :#: (f x)
                   p = Turn angle
                   n = Turn (- angle)

-- 5. arrowhead
arrowhead :: Int -> Command
arrowhead x = f x
              where
                f 0 = Go 10
                f (x+1) = (g x) :#: p :#: (f x) :#: p :#: (g x)
                g (x+1) = (f x) :#: n :#: (g x) :#: n :#: (f x)
                g 0 = Go 10
                n = Turn 60
                p = Turn (-60)

-- 6. snowflake
snowflake :: Int -> Command
snowflake x = (f x):#:  n :#: n  :#: (f x) :#: n  :#: n  :#: (f x)  :#: n :#: n
              where
                f 0 = Go 10
                f (x+1) = (f x) :#: p :#: (f x) :#: n :#: n :#: (f x) :#: p :#: (f x)
                n = Turn 60
                p = Turn (-60)

-- 7. hilbert
hilbert :: Int -> Command 
hilbert x =  l x
            where
              l 0 = Go 10
              l (x+1) = GrabPen (Colour 0.8 0.8 0.8):#: p :#: (r x) :#: (f x) :#: n :#: (l x) :#: (f x) :#: (l x) :#: n :#: (f x) :#: (r x) :#: p
              r 0 = Go 5
              r (x+1) = n :#: (l x) :#: (f x) :#: p :#: (r x) :#: (f x) :#: (r x) :#: p :#: (f x) :#: (l x) :#: n
              f 0 = Go 7
              f (x + 1) = Go 2 :#: Turn 1
              n = Turn 90
              p = Turn (- 90)



--tree
ltree :: Int -> Command
ltree x = f x
  where
  f 0       = GrabPen red :#: Go 10
  f (x+1) = g x :#: Branch (n :#: f x)
                 :#: Branch (p :#: f x)
                 :#: Branch (g x :#: f x)
  g 0       = GrabPen blue :#: Go 10
  g (x+1) = g x :#: g x
  n        = Turn 45
  p        = Turn (-45)

